@@ -232,7 +232,7 @@ int DumpPrometheusMetricsToIOBuf(butil::IOBuf* output) {
 
     if (bvar::FLAGS_bvar_max_dump_multi_dimension_metric_number > 0) {
         PrometheusMetricsDumper dumper_md(&os, g_server_info_prefix);
-        const int ndump_md = bvar::MVariable::dump_exposed(&dumper_md, NULL);
+        const int ndump_md = bvar::MVariableBase::dump_exposed(&dumper_md, NULL);
         if (ndump_md < 0) {
             return -1;
         }
