@@ -168,6 +168,11 @@ static void GlobalRelease() {
     }
 }
 
+void* UserExtendBlockPool(void* region_base, size_t region_size,
+                          int block_type) {
+    return ExtendBlockPoolByUser(region_base, region_size, block_type);
+}
+
 uint32_t RdmaRegisterMemory(void* buf, size_t size) {
     // Register the memory as callback in block_pool
     // The thread-safety should be guaranteed by the caller
@@ -213,7 +218,7 @@ static void FindRdmaLid() {
 
 static bool FindRdmaGid(ibv_context* context) {
     bool found = false;
-    for (int i = 0; i < g_gid_tbl_len; ++i) {
+    for (int i = g_gid_tbl_len - 1; i >= 0; --i) {
         ibv_gid gid;
         if (IbvQueryGid(context, g_port_num, i, &gid) < 0) {
             continue;
@@ -229,15 +234,15 @@ static bool FindRdmaGid(ibv_context* context) {
         // For infiniband, there is only one GID for each port.
         // For RoCE, there are 2 GIDs for each MAC and 2 GIDs for each IP.
         // Generally, the last GID is a RoCEv2-type GID generated by IP.
-        g_gid = gid;
-        g_gid_index = i;
-        found = true;
-    }
-    if (FLAGS_rdma_gid_index >= 0) {
-        if (g_gid_index != FLAGS_rdma_gid_index) {
-            found = false;
+        if (!found) {
+            g_gid = gid;
+            g_gid_index = i;
+            found = true;
         }
     }
+    if (FLAGS_rdma_gid_index >= 0 && g_gid_index != FLAGS_rdma_gid_index) {
+        found = false;
+    }
     return found;
 }
 
@@ -319,7 +324,7 @@ static void OnRdmaAsyncEvent(Socket* m) {
             break;
         }
         default:
-            // should not hannen
+            // should not happen
             IbvAckAsyncEvent(&event);
             break;
         }
@@ -337,16 +342,22 @@ static void OnRdmaAsyncEvent(Socket* m) {
     }
 
 static int ReadRdmaDynamicLib() {
-    g_handle_ibverbs = dlopen("libibverbs.so", RTLD_LAZY);
-    if (!g_handle_ibverbs) {
-        LOG(WARNING) << "Failed to load libibverbs.so " << dlerror() << " try libibverbs.so.1";
-        // Clear existing error
-        dlerror();
-        g_handle_ibverbs = dlopen("libibverbs.so.1", RTLD_LAZY);
-        if (!g_handle_ibverbs) {
-            LOG(ERROR) << "Fail to load libibverbs.so.1 due to " << dlerror();
-            return -1;
+    const static char* const kRdmaLibs[] = {
+        "libibverbs.so",
+        "libibverbs.so.1"
+    };
+    for (const char* lib : kRdmaLibs) {
+        dlerror();  // Clear existing error
+        g_handle_ibverbs = dlopen(lib, RTLD_LAZY);
+        if (g_handle_ibverbs) {
+            LOG(INFO) << "Successfully loaded " << lib;
+            break;
         }
+        LOG(WARNING) << "Failed to load " << lib << ": " << dlerror();
+    }
+    if (!g_handle_ibverbs) {
+        LOG(ERROR) << "Failed to load any of the RDMA libraries";
+        return -1;
     }
 
     LoadSymbol(g_handle_ibverbs, IbvGetDeviceList, "ibv_get_device_list");
@@ -686,6 +697,21 @@ bool SupportedByRdma(std::string protocol) {
     return false;
 }
 
+bool InitPollingModeWithTag(bthread_tag_t tag,
+                            std::function<void(void)> callback,
+                            std::function<void(void)> init_fn,
+                            std::function<void(void)> release_fn) {
+    if (RdmaEndpoint::PollingModeInitialize(tag, callback, init_fn,
+                                            release_fn) == 0) {
+        return true;
+    }
+    return false;
+}
+
+void ReleasePollingModeWithTag(bthread_tag_t tag) {
+    RdmaEndpoint::PollingModeRelease(tag);
+}
+
 }  // namespace rdma
 }  // namespace brpc
 
